clear, clc
close all

%% load results from init_poisson.m

load poisson_editing_setup

%% differential operators

% insert computation of gradient and Laplacian on Omega_0 here
% note: Laplacian has to be aware of boundary conditions!
N0 = h0*w0;
e = ones(N0,1);
Dx = spdiags([-e e],[0 h0],N0,N0);
Dx(N0-h0+1:N0,:) = 0;
Dy = spdiags([-e e],[0 1],N0,N0);
Dy(h0:h0:N0,:) = 0;
Laplacian = -Dx'*Dx-Dy'*Dy;

% indices of boundary pixels on Omega_0
boundary = [2:h0-1];                % left
boundary = [boundary N0-h0+2:N0-1];	% right
boundary = [boundary 1:h0:N0-h0+1];	% top
boundary = [boundary h0:h0:N0];		% bottom
boundary = transpose(boundary);

% modify Laplacian
N_boundary = size(boundary,1);
Laplacian(boundary,:) = sparse(1:N_boundary,boundary,ones(N_boundary,1),N_boundary,N0);

%% compute resulting image u in region Omega_0

u = f0;
for channel = 1:channels
	c_g0 = double(g0(:,:,channel));

	% Poisson equation
    gradient_u_x = Dx*c_g0(:);
    gradient_u_y = Dy*c_g0(:);
	divergence_gradient_u = -Dx'*gradient_u_x-Dy'*gradient_u_y;

	c_f0 = double(f0(:,:,channel));
	divergence_gradient_u(boundary) = c_f0(boundary);

	% resulting image in current channel:
	c_u = Laplacian\divergence_gradient_u;
	u(:,:,channel) = reshape(c_u,h0,w0);
end

%% compute and display total resulting image

u_tot = f;
u_tot(y_0_min:y_0_max,x_0_min:x_0_max,:) = u;
figure('Name','Resulting image resultat (Poisson + Dirichlet)','Position',[0,0,L,H])
imagesc(u_tot)
axis equal
axis off